;***************************************************************************
;* Atmel AVR-microcontroller and Epson LC-Display-controller SED1374       *
;* for use with "1/4VGA"-Display (320*240 pixel) Samsung UG32-F05          *
;* Data input via Linear Technology LTC1298 12Bit ADC                      *
;* Data output via Linear Technology LTC1257 12Bit DAC                     *
;* Voltage regulators for +5V (LM2575T5.0), +33V (LT1172) -24V (79L24)     *
;* and +3,3V (LM1086-3.3) from Linear Technology and National Semiconductor*
;* PCB Layout : Eagle light 4.0                                            *
;* Christoph Kessler 2002 - 2005                        (db1uq@t-online.de)*
;***************************************************************************
.nolist
.include "8515def.inc"
.list
;***************************************************************************
;* register 0 ( special purpose: 16Bit-LPM-operation ):
;***************************************************************************
.def	LpmReg	= r00	; for table-operations
;***************************************************************************
;* register 1-15 ( no "immediate"-operations possible ):
;***************************************************************************
.def	Null	= r01	; always zero for easy 16 bit addition
.def	ClicFlg	= r02	; Flag for click-switch, $FF= was pressed / $00=not
.def	RotVal	= r03	; value from rotator switch
.def	DeltaF	= r04	; frequency step (Span)
.def	Menu	= r05	; rotator switch mode number
.def	reg06	= r06	; 
.def	reg07	= r07	; 
.def	reg08	= r08	; 
.def	ADCh1	= r09	; y1-value ADC-channel 1
.def	AD1old	= r10	; old y1-value, used for dot-joining
.def	ADCh2	= r11	; y2-value ADC-channel 2
.def	AD2old	= r12	; old y2-value, used for dot-joining
.def	MkrLo	= r13	; Marker X-position low byte
.def	MkrHi	= r14	; Marker X-position high byte
.def	TunVLo	= r15	; tuning voltage value low byte
;***************************************************************************
;* register 16-25 ( "immediate"-operations possible ):
;***************************************************************************
.def	TunVHi	= r16	; tuning voltage value high byte
.def	XaxisL	= r17	; x-coordinate for display 0...319 high byte
.def	XaxisH	= r18	; x-coordinate for display 0...319 low byte
.def	CentLo	= r19	; center frequency value low byte
.def	CentHi	= r20	; center frequency value high byte
.def	Cnt	= r21	; general temporary counter
.def	Temp1	= r22	; general temporary register
.def	Temp2	= r23	; general temporary register
;***************************************************************************
;* register 24    ( also usable as 16Bit-register ) 
;***************************************************************************
.def	TempL	= r24	; general temporary register (low)
.def	TempH	= r25	; general temporary register (high)
;***************************************************************************
;* register 26-31 ( 16Bit-register ) already defined in "inc"-file:
;***************************************************************************
;	XL	= r26	; pointer for character output low
;	XH	= r27	; pointer for character output high
;	YL	= r28	; pointer for display fading address low
;	YH	= r29	; pointer for display fading address high
;	ZL	= r30	; pointer for lpm operations low
;	ZH	= r31	; pointer for lpm operations high

;*************************************************************************
;* constants :
;*************************************************************************
; Port B connections:
.equ	SCK	= 7	; PortB Bit7 = input  serial programming clock
.equ	MISO	= 6	; PortB Bit6 = output serial programming data out
.equ	MOSI	= 5	; PortB Bit5 = input  serial programming data in
.equ	CsAdc	= 4	; PortB Bit4 = output chip select ADC
.equ	SDat	= 3	; PortB Bit3 = bidir  serial data ADC and DAC
.equ	Click	= 2	; PortB Bit2 = input  click switch input
.equ	T1	= 1	; PortB Bit1 = input  Timer 1 from divider
.equ	T0	= 0	; PortB Bit0 = output Flash-RAM /CS
; Port D connections:
.equ	RD	= 7	; PortD Bit7 = output read LCD controller
.equ	WR	= 6	; PortD Bit6 = output write LCD controller
.equ	SClk	= 5	; PortD Bit5 = output serial clock ADC and DAC
.equ	LdDac	= 4	; PortD Bit4 = output load DAC
.equ	IN1	= 3	; PortD Bit3 = input  rotator switch channel B
.equ	IN0	= 2	; PortD Bit2 = input  rotator switch channel A
.equ	TXD	= 1	; PortD Bit1 = output UART out
.equ	RXD	= 0	; PortD Bit0 = input  UART in
; constants for tuning voltage stepping:
.equ	MaxSpn	= 36	; 36 max value for DeltaF (= full span)
.equ	Cornr1	= $1E	; $1E first corner of tuning voltage
.equ	Cornr2	= $2F	; $2F second corner of tuning voltage
.equ	CentrL	= $14	; $14 initial value of CentLo for full span
.equ	CentrH	= $16	; $16 initial value of CentHi for full span
; constants for frequency counter:
.equ	TCorr	= 01	; correction of timebase in microseconds
.equ	If_Hi	= high(-9590)	; substract intermediate frequency 
.equ	If_Lo	= low(-9590)	; (479,5MHz/1024)*20,48msec =9590
; constant for gray level
.equ	Grymin	= $08	; minimum gray level (1 above max grid level)

;--------------------------- Initialization ------------------------------

;*************************************************************************
;* reset/interrupt-vectors:
;*************************************************************************
.cseg
	rjmp	Initial		; after RESET to main program
	rjmp	ExtInt0		; interrupt rotator switch channel A
	rjmp	ExtInt1		; interrupt rotator switch channel B
	nop			; Timer1_capture
	nop			; Timer1_compareA
	nop			; Timer1_compareB
	nop			; Timer1_overflow
	nop			; Timer0_overflow
	nop			; SPI transfer
	nop			; UART Rx
	nop			; UART Empty
	nop			; UART Tx
	nop			; Analog comparator
;*************************************************************************
;* after reset: initialise stack,ports,ext.SRAM,interrupts 
;*************************************************************************
Initial: 
	ldi	Temp1,Low(RamEnd)
	out	spl,Temp1	;
	ldi	Temp1,High(RamEnd)
	out	sph,Temp1	; stack initialised
	clr	Null		; always zero (useful for 16+8 bit add.)
	out	PortA,Null	;
	out	PortB,Null	;
	out	PortC,Null	;
	out	PortD,Null	;
	out	DDRA,Null	; (dont care,display-RAM)
	out	DDRC,Null	; (dont care,display-RAM)
	ldi	Temp1,$11	; PortB direction = % 0001 0001
	out	DDRB,Temp1	; 
	ldi	Temp1,$F2	; PortD direction = % 1111 0010
	out	DDRD,Temp1	;
	ldi	Temp1,$CA	; ext.SRAM enable, add.waitstate,Int0/1 _/~
	out	MCUCR,Temp1	; MicroControllerUnit Control Reg.
	sbi	PortB,CsAdc	; ADC-/Enable _/~
	sbi	DDRB,SDat	; SDat = output
	clr	RotVal		;
	clr	ClicFlg		;
	ldi	YH,$40		; init counter for display-fading
	clr	YL
	clr	Menu		; init rotator switch mode
	ldi	CentLo,CentrL	; set Center frequency to initial value
	ldi	CentHi,CentrH	; = center of full span range
	ldi	Temp1,MaxSpn	; initialize full span
	mov	DeltaF,Temp1	;
	ldi	Temp1,160	; set marker position to center of screen
	mov	MkrLo,Temp1	;
	clr	MkrHi		;
;*************************************************************************
;* initialize LCD-controller registers
;*************************************************************************
	ldi	XH,$3F		; LCD-controller register 01 = Adr.$3FE1
	ldi	XL,$E1		;
	ldi	Cnt,28		; initialize 28 registers
	ldi	ZH,high(RegTbl)	; 
	ldi	ZL,low(RegTbl)	;
	lsl	ZL		; shift 1 left (16Bit memory addressing)
	rol	ZH		;
RgLoop:
	lpm
	st	X+,LpmReg	;
	adiw	ZL,1		; increment pointer
	dec	Cnt
	brne	RgLoop
;*************************************************************************
;* clear display
;*************************************************************************
	ldi	XH,$40		; pixel data ram.$4000-$DFFF, visible -D5FF
	ldi	XL,$00		;
Fillmem:
	st	X+,Null		; 2 white pixels
	cpi	XH,$E0		; End of RAM =$DFFF reached?
	brne	FillMem
;*************************************************************************
;* draw grid
;*************************************************************************
	ldi	XH,$40		; pixel data ram.$4000-$DFFF, visible -D5FF
	ldi	XL,$05		; start 12 pixel from bottom right
	ldi	TempH,high(159)	; go two lines (160byte each) up 
	ldi	TempL,low(159)	; = proceed 320 byte - 2 pixel
	ldi	Temp1,(16*GryMin-1)	; 1 grid and 1 white pixel
Grid1:
	ldi	Temp2,14	; draw 14 vertical lines
Grid2:
	st	X,Temp1		;
	adiw	XL,11		; one vertical grid line every 23 pixels
	dec	Temp2		;
	swap	Temp1		;
	st	X,Temp1		;
	adiw	XL,12		; one vertical grid line every 23 pixels
	swap	Temp1		;
	dec	Temp2		;
	brne	Grid2		;

	add	XL,TempL	; draw every second pixel only
	adc	XH,TempH	; 160 byte / line -> step up two lines
	cpi	XH,$D6		;
	brne	Grid1
;stop:
;	rjmp stop

;*************************************************************************
;* initialize text line above scope screen
;*************************************************************************
	ldi	XH,$D5		; upper right corner of display= $D560
	ldi	XL,$63		; (53char*6pixel)+2pixel=320pixel=160byte
	ldi	Cnt,53		; write 53 characters
	ldi	ZH,high(TxtTbl)	; output is from right to left
	ldi	ZL,low(TxtTbl)	; go to start of text-table
	lsl	ZL		; shift 1 left (16Bit memory addressing)
	rol	ZH		;
	adiw	ZL,52		; go to end of text (0..52 = 53 char)
TxLoop:
	lpm			; ASCII-char in LpmReg
	rcall	CharOut		; output character at position XH/XL
	sbiw	ZL,1		; decrement pointer
	dec	Cnt		;
	brne	TxLoop		;

	clr	ZL		; init start of printer-RAM $D600
	ldi	ZH,$D6		;

;*************************************************************************
;* start rotator switch interrupt 
;*************************************************************************
	ldi	Temp1,$C0	; set bit 7 and 6 
	out	GIMSK,Temp1	; in General Interupt MaSK register
	sei			; enable interrupts
	rjmp	Entry		; now start main loop

;--------------------------- Main Program loop ---------------------------

;*************************************************************************
;* part of main loop (positioned here for short branch <64 byte):
;* handling of click-switch, Entry = entry point from init routine
;*************************************************************************
Clic2:
	sbis	PinB,Click	; was pressed, is it released now ?
	rjmp	Cycle		; no, so forget rotator value
	ldi	Temp1,4		; max Menu number =3
	inc	Menu		; next Menu
	cp	Menu,Temp1	; max reached?
	brlo	Entry		; no
	clr	Menu		; 
Entry:	
	push	XH		; save X
	push	XL		;
	push	ZH		; save Z
	push	ZL		;
	ldi	XH,$D1		; left end of underline +1 =$D1A0
	ldi	XL,$A0		;
Clear:
	st	-X,Null		; clear underline
	tst	XL		; right end reached?
	brne	Clear		; no
	ldi	ZH,high(MnuTbl)	; fetch length and position of underline
	ldi	ZL,low(MnuTbl)	;
	add	ZL,Menu		; =0..3 (will be multiplied by 2)
	adc	ZH,Null		; add carry
	lsl	ZL		; shift left for 16bit addressing
	rol	ZH		;
	lpm			; fetch number of bytes
	mov	Cnt,LpmReg	; into byte counter
	adiw	ZL,1		; proceed 1 byte
	lpm			; fetch X-position of most left byte
	mov	XL,LpmReg	; XH is still = $D1
	ldi	Temp1,$FF	; 2 black pixels 
Underl:
	st	-X,Temp1	; one byte of underline
	dec	Cnt		; number reached?
	brne	Underl		;
	pop	ZL		; restore Z
	pop	ZH		;
	pop	XL		; restore X
	pop	XH		;
	clr	ClicFlg		; reset flag
	rjmp	Cycle		; forget rotator, return to main loop
Clic3:
	com	ClicFlg		; set flag to $FF
	rjmp	Cycle		; forget rotator, return to main loop
;*************************************************************************
;* handling of rotator-switch value: branch to current menu routine
;*************************************************************************
Menu0:
	cli			; disable interrupt
	mov	TempL,RotVal	; 
	clr	RotVal		; 
	sei			; enable interrupt
	mov	Temp1,Menu	; 
	cpi	Temp1,0		; 
	brne	Menu1		;
	rjmp	Center		; "0"= change center frequency	
Menu1:
	cpi	Temp1,1		; 
	brne	Menu2		;
	rjmp	Span		; "1"= change frequency span	
Menu2:
	cpi	Temp1,2		; 
	brne	Menu3		;
	rjmp	Marker		; "2"= change marker frequency	
Menu3:
	cpi	Temp1,3		; 
	brne	Clic2		; must be an error
	rjmp	Print		; "3"= print hardcopy	

;*************************************************************************
;* main loop entry point: continuous triangular sweep cycles,
;* between cycles: update rotator switch and click switch 
;*************************************************************************
Main:
	tst	ClicFlg		; click switch was pressed last time?
	brne	Clic2		; 
	sbis	PinB,Click	; pressed now the first time ?
	rjmp	Clic3		; then set Click-Flag for next time

	tst	RotVal		; rotator switch changed ?
	brne	Menu0		; then update value
;*************************************************************************
;* sweep one cycle = 2* 319 steps, start rising from center frequency
;*************************************************************************
Cycle:
	ldi	XaxisL,160	; x-coordinate = 160
	clr	XaxisH		;
	mov	TunVLo,CentLo	; tuning voltage set to center frequency
	mov	TunVHi,CentHi	;

Rise1:
	rcall	ADDA		; read/display old Y, set new tuning voltage 
	inc	XaxisL		; increment X-coordinate low byte
	breq	IncHi		;
	cpi	XaxisL,$3F	; 319 dec. = $13F
	breq	Fall3		;
Rise2:
	cpi	TunVHi,Cornr1	; first corner voltage reached ?
	brlo	Rise4		; no, just single increment
	cpi	TunVHi,Cornr2	; second corner voltage reached ?
	brlo	Rise3		; no, just double increment
	add	TunVLo,DeltaF	; yes, increment voltage three times
	adc	TunVHi,Null	; (add carry flag only)
Rise3:
	add	TunVLo,DeltaF	; increment voltage
	adc	TunVHi,Null	; (add carry flag only)
Rise4:
	add	TunVLo,DeltaF	; increment voltage
	adc	TunVHi,Null	; (add carry flag only)
	rjmp	Rise1		; 

Fall1:
	rcall	ADDA		; read/display old Y, set new tuning voltage
	cp	MkrLo,XaxisL	; marker position reached?
	brne	Fall2		;
	cp	MkrHi,XaxisH	;
	brne	Fall2		;
	rjmp	FCount		; yes, count frequency
Fall2:
	dec	XaxisL
	cpi	XaxisL,$FF
	breq	DecHi		; XaxisL = $FF ? -> add carry to XaxisH
	tst	XaxisL		; XaxisL = 0 ?
	brne	Fall3		; 
	tst	XaxisH		; = $000 or $100 ?
	breq	Rise6		; = $000 -> now rise from 0 to 159
Fall3:
	sub	TunVLo,DeltaF	; decrement voltage
	sbc	TunVHi,Null	; (subtract carry flag only)
	cpi	TunVHi,$C0	; below zero?
	brsh	Fall1		; yes, just substract once
	cpi	TunVHi,Cornr1	; first corner voltage reached ?
	brlo	Fall1		; 
	sub	TunVLo,DeltaF	; decrement voltage
	sbc	TunVHi,Null	; (subtract carry flag only)
	cpi	TunVHi,Cornr2	; second corner voltage reached ?
	brlo	Fall1		; 
	sub	TunVLo,DeltaF	; decrement voltage
	sbc	TunVHi,Null	; (subtract carry flag only)
	rjmp	Fall1		;

Rise5:
	rcall	ADDA		; read/display old y, set new tuning voltage
	inc	XaxisL		; increment X-coordinate low byte
	cpi	XaxisL,160	; center reached?
	breq	Rise9		; yes, sweep cycle ends
Rise6:
	cpi	TunVHi,$C0	; below zero?
	brsh	Rise8		; yes, just add once
	cpi	TunVHi,Cornr1	; first corner voltage reached ?
	brlo	Rise8		; no
	cpi	TunVHi,Cornr2	; second corner voltage reached ?
	brlo	Rise7		; no
	add	TunVLo,DeltaF	; increment voltage
	adc	TunVHi,Null	; (add carry flag only)
Rise7:
	add	TunVLo,DeltaF	; increment voltage
	adc	TunVHi,Null	; (add carry flag only)
Rise8:
	add	TunVLo,DeltaF	; increment voltage
	adc	TunVHi,Null	; (add carry flag only)
	rjmp	Rise5		; 
Rise9:
	adiw	ZL,2		; 2*319 values +2 = 640 =$280
	cpi	ZH,$E0		; end of printer-RAM $E000 reached?
	brlo	Rise10		; no
	clr	ZL		; goto start of printer-RAM $D600
	ldi	ZH,$D6		;
Rise10:
	rjmp	Main		; start new cycle

IncHi:
	inc	XaxisH		;
	rjmp	Rise2		;
DecHi:
	dec	XaxisH		;
	rjmp	Fall3		;

;--------------------------- Subroutine ADDA -----------------------------

;*************************************************************************
;* read y-value (8 Bit) serially from  AD-converter, set pixel
;*************************************************************************
ADDA:
	sbi	PortD,SClk	; DAC-Clock _/~
	sbi	DDRB,SDat	; SDat = output
	sbi	PortB,SDat	; SDat = high
	cbi	PortB,CsAdc	; ADC-/Enable ~\_
	rcall	ADClk		; (3) Startbit =high
	rcall	ADClk		; (3) SGL =high -> 2 inputs
	rcall	ADClk		; (3) odd = high -> input nr.2
	rcall	ADClk		; (3) MSB first = high
	cbi	DDRB,SDat	; SDat = input
	rcall	ADClk		; (3) one unused bit in the beginning
	mov	AD2old,ADCh2	; save old AD-value for dot-joning
	clr	ADCh2		; 
	ldi	Cnt,8		; counter, 8 of 12 Bit used
ADDA1:
	rcall	ADClk		; (3)
	clc			; 
	sbic	PinB,SDat	; skip next line if SDat= 0
	sec			; SDat=1 so Cy=1
	rol	ADCh2		; shift SDat into lowest bit of ADCh2
	dec	Cnt		; 
	brne	ADDA1		;  
;	ldi	Cnt,4		; counter for unused 4 bit 
;Rest4:
;	rcall	ADClk		; (3)
;	dec	Cnt		; 
;	brne	Rest4		; 
	sbi	PortB,CsAdc	; ADC-/Enable _/~
	sbi	DDRB,SDat	; SDat = output
	cbi	PortD,SClk	; DAC-Clock ~\_
;*************************************************************************
;* tune voltage controlled oscillator to next frequency 
;*************************************************************************
	mov	TempH,TunVHi	; load DAC-value
	mov	TempL,TunVLo	;
	lsl	TempL		; shift into Cy, MSB first
	rol	TempH		;
	brcs	Overfl		; MSB set --> over- or underflow
	lsl	TempL		; 2nd MSB into Cy
	rol	TempH		;
	brcs	Overf1		; MSBs = %01 -->overflow
;*************************************************************************
;* output 12 Bit serially to D/A-converter LTC1257 
;*************************************************************************
ADDA2:
	ldi	Cnt,12
	lsl	TempL		; DAC-MSB into Cy
	rol	TempH		; 
ADDA3:
	brcc	ADDA4		; one bit via Cy into DAC
	sbi	PortB,SDat	; SDat=H
	brcs	ADDA5
ADDA4:
	cbi	PortB,SDat	; SDat=L
ADDA5:
	sbi	PortD,SClk	; DAC-clock _/~
	lsl	TempL
	rol	TempH		; max clock freq. 1,4MHz!
	dec	Cnt		; does not change Cy
	cbi	PortD,SClk	; DAC-clock ~\_
	brne	ADDA3		; 12 bit reached ?
	cbi	PortD,LdDac	; DA-converter /load ~\_
	nop
	sbi	PortD,LdDac	; DA-converter /load _/~
;*************************************************************************
;* now store ADC-value(s) and set pixel(s)
;*************************************************************************
	st	Z+,ADCh2	; save original ADC-value ADCh2 for printer
	ldi	Temp1,230	;
	cp	Temp1,ADCh2	; limit ADCh2-value to 0...230 for display
	brsh	ADDA6		; 
	mov	ADCh2,Temp1	;
ADDA6:
	cp	ADCh2,AD2old	; which y-value is larger ?
	brsh	ADDA7		; old value was smaller
	mov	Temp1,AD2old	; old y-value into Temp1
	mov	Cnt,AD2old	;
	sub	Cnt,ADCh2	; difference (old-new) into counter
	rjmp	ADDA8		;
ADDA7:
	mov	Temp1,ADCh2	; new y-value into Temp1
	mov	Cnt,ADCh2	;
	sub	Cnt,AD2old	; difference (new-old) into counter
ADDA8:
	rcall	DrawPx		; set pixel at x=XaxisH/XaxisL,y=Temp1
	ret			; 
;*************************************************************************
;* handling of overflow (> 4095) / underflow (< 0) of tuning voltage
;*************************************************************************
Overfl:
	lsl	TempL		; fetch 2nd overflow-bit
	rol	TempH		;
	brcs	Underf		; MSBs = %11 --> underflow
Overf1:
	ldi	TempL,$FC	; MSBs= %10 or %01 --> overflow
	ldi	TempH,$FF	; DAC = 4095, max tuning voltage
	rjmp	ADDA2		;
Underf:
	clr	TempL		; DAC = 00, min tuning voltage
	clr	TempH		;
	rjmp	ADDA2		;


;--------------------------- Subroutine DrawPx ----------------------------

;*************************************************************************
;* subroutine: draw vertical line of pixel(s) in display RAM,
;* top y-value in Temp1, x = XaxisH/XaxisL, 
;* Cnt = number of pixels down,  Cnt = 0 --> set single pixel
;*************************************************************************
DrawPx:
	push	XL		; 
	push	XH		; 
	mov	XL,Temp1	; Temp1 * 160
	clr	XH		;
	lsl	XL		; 5* shift left -> Temp1*32
	rol	XH		;
	lsl	XL		;
	rol	XH		;
	lsl	XL		;
	rol	XH		;
	lsl	XL		;
	rol	XH		;
	lsl	XL		;
	rol	XH		;
	mov	TempL,XL	;   1*32
	mov	TempH,XH	;
	lsl	XL		; 
	rol	XH		;
	lsl	XL		;
	rol	XH		; + 4*32
	add	XL,TempL	;  
	adc	XH,TempH	; = 5*32=160
	subi	XH,-$40		; add $40 -> pixel-ram starts at $4000
	mov	TempH,XaxisH	; XaxisH,XaxisL /2 (2Pixel/Byte)
	mov	TempL,XaxisL
	lsr	TempH		; shift right
	ror	TempL		; 9bit -> 8 bit in TempL, LSB in Cy
	ldi	TempH,$F0	; Cy = low -> set left Pixel (or $F0)
	brcc	DrwPx1		; Cy = high -> set right Pixel (or $0F)
	swap	TempH		; $F0 -> $0F
;*************************************************************************
;* coordinate transformation: origin at bottom right to bottom left
;* display coordinates in decimal values x,y:
;*  top left:    159,239   top right:      0,239     
;*  bottom left: 159,  0   bottom right:   0,  0
;* transformed coordinates:
;*  top left:      0,239   top right:    159,239
;*  bottom left:   0,  0   bottom right: 159,  0	
;*************************************************************************
DrwPx1:
	com	TempL		; $00->$FF, $9F->$60
	subi	TempL,$60	; $FF->$9F, $60->$00  (255-159=$60) 
	add	XL,TempL	; Pixel-addr:=$4000+(160*Y)+(160-X/2)
	adc	XH,Null		; add carry only

	ldi	Temp1,17*(GryMin-1)	; we dont want to change grid pixels
	and	Temp1,TempH	; Temp1 = grid value = $07 or $70
DrwPx2:
	ld	TempL,X		; fetch 2 pixel from ram into TempL
	mov	Temp2,TempH	; Temp2 = pixel mask = $0F or $F0
	and	Temp2,TempL	; Temp2 = wanted pixel, unwanted pixel=0
	cp	Temp1,Temp2	; compare wanted pixel with grid value
	breq	DrwPx3		; 	
	or	TempL,TempH	; set wanted pixel, unwanted pixel unchanged
	st	X,TempL		; back to ram
	dec	Cnt		; bottom end of line reached?
	brmi	DrwPx3		; Cnt=$FF -> end of line 
	sbiw	XL,50		;
	sbiw	XL,50		; 160 byte back in display-ram
	sbiw	XL,60		; (sbiw max 63 allowed)
	cpi	XH,$40		; outside display-ram?
	brlo	DrwPx3		; yes
	rjmp	DrwPx2		; no, draw next pixel below last one
DrwPx3:
	pop	XH		;
	pop	XL		;
	ret			;

;--------------------------- Subroutine ADCLk ----------------------------

;*****************************************************************************
;* subroutine: one A/D-clock for LTC1298 (f-max.=250kHz).Delay time is used 
;* to fade up to 4 gray pixels for one gray level or test up to 8 white pixels
;*****************************************************************************
ADClk:
	ld	TempL,Y		; (2) wait at least 2 usec for ADC
	cpi	TempL,Grymin	; (1)  load 1 byte,  2 empty or grid pixels?
	brsh	ADCk20		; (1+1) > $07 -> decrement(s) necessary
	adiw	YL,1		; (2)   = $07..$00 -> no change, fetch next
	cpi	YH,$D1		; (1) Scope-RAM end $D0FF reached ?
	breq	ADCk10		; (1+1) then init to $4000
ADCk01:
	ld	TempL,Y		; (2) time enough for same procedure once again
	cpi	TempL,Grymin	; (1)  load 1 byte,  2 empty or grid pixels?
	brsh	ADCk20		; (1+1) > $07 -> decrement(s) necessary
	adiw	YL,1		; (2)   = $07..$00 -> no change, fetch next
ADCk02: 
	cpi	YH,$D1		; (1) Scope-RAM end $D0FF reached ? 
	breq	ADCk11		; (1+1) then init to $4000
ADCk03:
	cbi	PortD,SClk	; (2) ADC-Clock ~\_
	ld	TempL,Y		; (2) wait at least 2 usec for ADC
	cpi	TempL,Grymin	; (1)  load 1 byte,  2 empty or grid pixels?
	brsh	ADCk30		; (1+1) > $07 -> decrement(s) necessary
	adiw	YL,1		; (2)   = $07..$00 -> no change, fetch next
	cpi	YH,$D1		; (1) Scope-RAM end $D0FF reached ?
	breq	ADCk12		; (1+1) then init to $4000
ADCk04:
	ld	TempL,Y		; (2) time enough for same procedure once again
	cpi	TempL,Grymin	; (1)  load 1 byte,  2 empty or grid pixels?
	brsh	ADCk30		; (1+1) > $07 -> decrement(s) necessary
	adiw	YL,1		; (2)   = $07..$00 -> no change, fetch next
ADCk05:
	cpi	YH,$D1		; (1) Scope-RAM end $D0FF reached ? 
	breq	ADCk13		; (1+1) then init to $4000
ADCk06:
	sbi	PortD,SClk	; (2) ADC-Clock _/~
	ret			; (4)
;*************************************************************************
; goto start of display ram
;*************************************************************************
ADCk10:
	ldi	YH,$40		; (2) start address =$4000
	clr	YL		; (1)
	rjmp	ADCk01		; (2)
ADCk11:
	ldi	YH,$40		; (2) start address =$4000
	clr	YL		; (1)
	rjmp	ADCk03		; (2)
ADCk12:
	ldi	YH,$40		; (2) start address =$4000
	clr	YL		; (1)
	rjmp	ADCk04		; (2)
ADCk13:
	ldi	YH,$40		; (2) start address =$4000
	clr	YL		; (1)
	rjmp	ADCk06		; (2)

;*************************************************************************
; decrement both gray nibbles towards zero, but omit gridlevel $01 
;*************************************************************************
ADCk20:
	mov	TempH,TempL	; (1)
	andi	TempL,$0F	; (1) test low nibble
	cpi	TempL,Grymin	; (1) 
	breq	ADCk23		; (1+1) = x8     -> low nibble = 0
	brlo	ADCk21		; (1+1) = x7..x0 -> low nibble unchanged
	dec	TempL		; (1) fade low nibble
ADCk21:
	andi	TempH,$F0	; (1) test high nibble
	cpi	TempH,16*Grymin	; (1) 
	breq	ADCk24		; (1+1) = 8x     -> high nibble = 0
	brlo	ADCk22		; (1+1) = 7x..0x -> high nibble unchanged
	subi	TempH,$10	; (1) fade high nibble
ADCk22:
	or	TempL,TempH	; (1) and add both nibbles again
	st	Y+,TempL	; (2) back to display ram
	rjmp	ADCk02		; (2)
;*************************************************************************
; from 2 jump to 0 omitting 1
;*************************************************************************
ADCk23:
	clr	TempL		; (1)
	rjmp	ADCk21		; (1)
ADCk24:
	clr	TempH		; (1)
	rjmp	ADCk22		; (1)
;*************************************************************************
; same routine, we avoid a delay of 7 clocks for subroutine call/ret 
;*************************************************************************
ADCk30:
	mov	TempH,TempL	; (1)
	andi	TempL,$0F	; (1) test low nibble
	cpi	TempL,Grymin	; (1) 
	breq	ADCk33		; (1+1) = x8     -> low nibble = 0
	brlo	ADCk31		; (1+1) = x7..x0 -> low nibble unchanged
	dec	TempL		; (1) fade low nibble
ADCk31:
	andi	TempH,$F0	; (1) test high nibble
	cpi	TempH,16*Grymin	; (1) 
	breq	ADCk34		; (1+1) = 8x     -> high nibble = 0
	brlo	ADCk32		; (1+1) = 7x..0x -> high nibble unchanged
	subi	TempH,$10	; (1) fade high nibble
ADCk32:
	or	TempL,TempH	; (1) and add both nibbles again
	st	Y+,TempL	; (2) back to display ram
	rjmp	ADCk05		; (2)
;*************************************************************************
; from 2 jump to 0 omitting 1
;*************************************************************************
ADCk33:
	clr	TempL		; (1)
	rjmp	ADCk31		; (1)
ADCk34:
	clr	TempH		; (1)
	rjmp	ADCk32		; (1)

;--------------------------- Subroutine CharOut ---------------------------

;*****************************************************************************
;* subroutine: display ASCII-character at position XH/XL 
;* =left byte,top line of 7 lines, ASCII-Char is in LpmReg
;* X-Position increments from right to left for easier number output
;*****************************************************************************
CharOut:
	push	XL		; save X
	push	XH
	push	ZL		; save Z
	push	ZH
	push	Cnt		; save Cnt
	push	TempH		;
	push	TempL		;
	ldi	Cnt,7		; 8 lines * 3 Byte = 1 character
	clr	ZH		; 
	adiw	XL,1		; we use "st -X" but we need "st X-"
	mov	ZL,LpmReg	; ASCII char into Z
	sbiw	ZL,$20		; no control characters, first char is $20
	lsl	ZL		; table has 8 byte / char.
	rol	ZH		; char. * 8 = position from start of table
	lsl	ZL		; last *2 after addition
	rol	ZH
	subi	ZL,low(-Tab5x8)	; add 16bit words by subtracting negatives
	sbci	ZH,high(-Tab5x8)
	lsl	ZL		; shift 1 left (16Bit memory addressing)
	rol	ZH		; 
CharLoop:
	lpm			; fetch 2 pixels of char
	ldi	TempH,3		; move 3 * 2 pixel  4bit into 3 byte
PixLoop:
	lsl	LpmReg		; shift left pixel into carry
	ror	TempL		; carry into MSB
	asr	TempL		; and also next 3 bits
	asr	TempL		; will be low nibble
	asr	TempL
	lsl	LpmReg		; shift right pixel into carry
	ror	TempL		; carry into MSB
	asr	TempL		; and also next 3 bits
	asr	TempL		; will be high nibble
	asr	TempL
	st	-X,TempL	; 2 pixel to display-ram
	dec	TempH		; 3 bytes filled ?
	brne	PixLoop
	
	adiw	ZL,1		; next 6 pixel in table Tab5x8
	sbiw	XL,50		; next display line (160-3) byte back
	sbiw	XL,50		; 
	sbiw	XL,57		; (sbiw: max 63 allowed)

	dec	Cnt
	brne	CharLoop

	pop	TempL		;
	pop	TempH		;
	pop	Cnt		; restore old Cnt
	pop	ZH		; restore old Z
	pop	ZL
	pop	XH		; restore old X
	pop	XL
	adiw	XL,3		; move to next char. position 6pixel=3byte
	cpi	XL,$00		; start of text line = ($D5FF+1) reached ?
	brne	CharOu1		; no so return
	ldi	XL,$63		; go to end of text line =$D560+3
CharOu1:
	ret
;--------------------------- Rotator switch value update -----------------
;
;*************************************************************************
;* change center frequency, tuning speed depending from span
;* multiply RotVal at least *4
;*************************************************************************
Center:
	ldi	Cnt,8		; simple log conversion for DeltaF
	mov	TempH,DeltaF	; 128..255 -> Cnt=7 --- 0..1 -> Cnt=0
Centr1:
	lsl	TempH		; shift DeltaF looking for leading "1"
	dec	Cnt		; 
	breq	Centr2		; DeltaF is 0 or 1
	brcc	Centr1		; leading "1" not yet reached ?
Centr2:
	cpi	TempL,$80	; increase or decrease center frequ.?
	brsh	Centr5		; $00..$7F=inc / $FF..$80=dec
	lsl	TempL		; RotVal *2 
Centr3:
	lsl	TempL		; RotVal *2
	dec	Cnt		; 
	brne	Centr3		;
	add	CentLo,TempL	; increase center frequency
	adc	CentHi,Null	; add Cy only
	cpi	CentHi,$40	; >$3FFF ?
	brlo	Centr4
	ldi	CentHi,$3F	; set to $3FFF
	ldi	CentLo,$FF	;
Centr4:
	rjmp	Cycle		; start next sweep cycle
Centr5:
	com	TempL		; $FF...$80 -> $00...$7F
	inc	TempL		;           -> $01...$80
	lsl	TempL		; RotVal *2 
Centr6:
	lsl	TempL		; RotVal *2
	dec	Cnt		; 
	brne	Centr6		;
	sub	CentLo,TempL	; decrease center frequency
	sbc	CentHi,Null	; substract Cy only
	brpl	Centr7		; <$0000 ?
	clr	CentHi		; set to $0000
	clr	CentLo		;
Centr7:
	rjmp	Cycle		; start next sweep cycle
;*************************************************************************
;* change frequency span
;*************************************************************************
Span:
	cpi	TempL,$80	; increase or decrease span?
	brsh	Span2		; $00..$7F=inc / $FF..$80=dec
	mov	TempL,DeltaF	; max span reached ?
	cpi	TempL,MaxSpn	;
	breq	Span1		; yes, don't increment
	inc	DeltaF		;
Span1:
	rjmp	Cycle
Span2:
	tst	DeltaF		; min span = 0 reached?
	breq	Span3		;
	dec	DeltaF		;
Span3:
	rjmp	Cycle		; start next sweep cycle
;*************************************************************************
;* change marker frequency
;*************************************************************************
Marker:
	rjmp	Cycle		; start next sweep cycle
;*************************************************************************
;* print hardcopy
;*************************************************************************
Print:
	push	ZH		; save Z
	push	ZL		;
	ldi	Temp1,25	; 8MHz/16*(25+1) = ca. 19200 Hz
	out	UBRR,Temp1	; into Uart-Baud-Rate-Register
	ldi	Temp1,$18	; set Rx/Tx-enable flags, no interrupts
	out	UCR,Temp1	; init Uart-Control-Register
	ldi	ZL,low(HpgTbl)	; init printer for HP-GL, print frame
	ldi	ZH,high(HpgTbl)	;
	ldi	Cnt,(2*(Tab5x8-HpgTbl))
	lsl	ZL		; *2 for 16bit addressing
	rol	ZH		;
Prnt1:
	lpm			;
	mov	Temp1,LpmReg	; 
	tst	Temp1		;
	breq	PCurve		; 
	rcall	SerOut		; output character to serial port
Prnt2:
	adiw	ZL,1		; next character
	dec	Cnt		; 
	brne	Prnt1		; 
	pop	ZL		; restore Z
	pop	ZH		;
	clr	RotVal		; dont print another time
	rjmp	Cycle		; start next sweep cycle

;*************************************************************************
;* print 4 curves stored in printer-ram
;*************************************************************************
PCurve:
	push	ZH		; save Z
	push	ZL		;
	ldi	ZH,$D6		; printer ram is $D600-$DFFF
	clr	ZL		;
PCurv1:
	ldi	Temp1,'P'	; move HPGL-"pen" to start of curve
	rcall	SerOut		;
	ldi	Temp1,'U'	; "Pen Up"
	rcall	SerOut		; 
;*************************************************************************
;* get X-value from ram address
;*************************************************************************
PCurv2:
	mov	TempH,ZH	;
	mov	TempL,ZL	;
	subi	TempH,$D6	; $D600 -> $0000, $DFFF -> $09FF
PCurv3:
	cpi	TempH,$02	; we have 4 curves, each has $027D points 
	brlo	PCurv5		; is < $0200
	brne	PCurv4		; is > $02FF
	cpi	TempL,$80	; 
	brlo	PCurv5		; is < $0280
PCurv4:
	subi	TempL,$80	; substract $0280
	sbci	TempH,$02	;
	rjmp	PCurv3		; loop until result is below $0280

PCurv5:
	cpi	TempH,$01	; subdivisions are 0-9F/A0-1DD/1DE-27D
	breq	PCurv6		; is = $01xx -> 2nd or 3rd part of curve
	brsh	PCurv8		; is > $01FF -> 3rd part
	cpi	TempL,$A0	; is < $0100, >= $00A0 ?
	brsh	PCurv7		; then -> 2nd part of curve

	ldi	Temp1,$A0	; 1st part of curve, add $A0
	add	TempL,Temp1	; X rising from 160 to 319 = $A0...$13F
	adc	TempH,Null	; (add carry only)
	rjmp	PCurv10		; ready to print x-value

PCurv6:
	cpi	TempL,$DE	; >= $01DE ?
	brsh	PCurv9		; -> 3rd part of curve
PCurv7:
	ldi	Temp1,$DE	; 2nd part of curve, substract $1DE-x
	sub	Temp1,TempL	;
	mov	TempL,Temp1	;
	ldi	Temp1,$01	; X falling from 318 to 1
	sbc	Temp1,TempH	;
	mov	TempH,Temp1	;
	rjmp	PCurv10		; ready to print x-value
PCurv8:
	cpi	Templ,$7E	; = unused addresses $027E/$027F ?
	brlo	PCurv9		; no belongs to 3rd part of curve
	adiw	ZL,2		; point to next multiple of $280
	ldi	Temp1,$0D	; 
	rcall	Serout		; print CR
	rjmp	PCurv1		; jump to next curve
PCurv9:
	subi	TempL,$DE	; 3rd part of curve, substract $1DE
	sbci	TempH,$01	; X rising from 0 to 159 = $000...$09F
;*************************************************************************
;* convert x-value -> *20 -> decimal -> ASCII -> plot HPGL 
;* 1 HPGL print unit = 1/40 mm -> 20* x-value = 0...159,5mm
;*************************************************************************
PCurv10:
	adiw	TempL,35	; offset left side =(35*20) = 700 *1/40mm
	lsl	TempL		; x-value *4
	rol	TempH		; 
	lsl	TempL		;
	rol	TempH		;
	push	TempH		; save x-value *4
	push	TempL		;
	lsl	TempL		;
	rol	TempH		; 
	lsl	TempL		;
	rol	TempH		; x-value *16
	pop	Temp1		; 
	add	TempL,Temp1	; add both numbers
	pop	Temp1		;
	adc	TempH,Temp1	; -> x-value*(16+4)
	rcall	DecOut		; output as 4 digit decimal ASCII char.

	ldi	Temp1,','	; 
	rcall	Serout		; print comma
;*************************************************************************
;* convert y-value -> *16 -> decimal -> ASCII -> plot HPGL 
;* 1 HPGL print unit = 1/40 mm -> 16* y-value = 0...102mm
;*************************************************************************
	ld	TempL,Z+	; fetch y-value
	clr	TempH		;
	adiw	TempL,20	; offset bottom side =(20*16)=320 *1/40mm
	lsl	TempL		;
	rol	TempH		;
	lsl	TempL		;
	rol	TempH		; * 16
	lsl	TempL		;
	rol	TempH		;
	lsl	TempL		;
	rol	TempH		;
	rcall	DecOut		; and output to printer

	mov	Temp1,ZL	; every 8 plots insert CR "PD"
	andi	Temp1,$07	;
	cpi	Temp1,$01	;
	brne	PCurv11
	ldi	Temp1,$0D
	rcall	Serout		; print CR
	ldi	Temp1,'P'	; 
	rcall	SerOut		;
	ldi	Temp1,'D'	; "Pen Down"
	rcall	SerOut		; 
	rjmp	PCurv2		;
PCurv11:
	cpi	ZH,$DA		; printer ram end = $DAFE reached?
	brlo	PCurv12		; no
	cpi	ZL,$FE		; 
	brlo	PCurv12		; no
	rjmp	PCurv13		; yes, end of curves
PCurv12:
	ldi	Temp1,','	; print comma
	rcall	SerOut		;
	rjmp	PCurv2		; ("relative branch out of reach")
PCurv13:
	pop	ZL		; restore Z
	pop	ZH		;
	rjmp	Prnt2

;*************************************************************************
;* count frequency at marker position
;*************************************************************************
FCount:
	push	ZH		; save Z
	push	ZL		;
	push	XH		; save X
	push	XL		;
	ldi	Temp1,220	; dont go above scope ram 
	cp	Temp1,ADCh2	; limit marker line to 0...230
	brlo	FCnt1		; ADCh2 is > 220
	mov	Temp1,ADCh2	; ADCh2 = 0..220
FCnt1: 
	subi	Temp1,-10	; go 10 pixel above curve
	ldi	Cnt,20		; marker has 20 vertical pixels
	rcall	DrawPx		; DrawPx is limited to < 231
FCnt2:
	ldi	TempL,If_Hi	; start values for Timer 1
	out	TCNT1H,TempL	; used to add or substract
	ldi	TempL,If_Lo	; intermediate frequency
	out	TCNT1L,TempL	;
	ldi	TempL,$07	; start Timer 1 counting rising pulses
	out	TCCR1B,TempL	; at T1 input = PortB bit 1
	ldi	TempL,TCorr	; correction of start and stop procedure
	out	TCNT0,TempL	; in microseconds, (use "nop" for 125nsec)
	ldi	Cnt,80		; 80*256 sec = 20,48 msec
	ldi	TempL,02	; 
	out	TCCR0,TempL	; start T0 counting Clock/8 = 1 MHz
FCnt3:
	in	TempL,TIFR	; 256 sec reached?
	sbrs	TempL,1		; bit 1 set?
	rjmp	FCnt3		; no, go back
	ori	TempL,$02	; set TOV0 in TIFR to reset flag
	out	TIFR,TempL	;
	dec	Cnt		; 80*256sec over ?
	brne	FCnt3		; no, go back
	out	TCCR1B,Null	; stop frequency counter
	out	TCCR0,Null	; stop timebase counter

	clr	ZH		; Z=counter/pointer to reg 01..reg05
	ldi	ZL,6		; display 5 BCD digits + "0.05-bit"
	ldi	XL,$81		; set X-Position for marker text
	ldi	XH,$D5		;
	in	TempL,TCNT1L	; fetch  low byte of 16bit word
	in	TempH,TCNT1H	; fetch high byte of 16bit word

	lsr	TempH		; divide by 2
	ror	TempL		; for "0,05"-digit
	ldi	Temp1,'0'	; 
	brcc	FCnt5		; Cy low="0", high="5"
	ldi	Temp1,'5'	; 
	rjmp	FCnt5		; output to display
FCnt4:
	rcall	Div10		; divide by 10, digit in Temp1
	ori	Temp1,$30	; convert one digit BCD to ASCII
FCnt5:
;	st	Z,Temp1		; save digits for printer -> reg06..reg01
	mov	LpmReg,Temp1	; via LpmReg at XH/XL
	rcall	CharOut		; output to display, next X-Position left
	dec	ZL		; 6 digits over?
	breq	FCnt6		; ok
	cpi	ZL,4		; decimal point position?
	brne	FCnt4		; no, go to next digit
	adiw	XL,3		; jump over decimal point
	rjmp	FCnt4		; then next digit
FCnt6:
	pop	XL		; restore X
	pop	XH		;
	pop	ZL		; restore Z
	pop	ZH		;
	rjmp	Fall2		; 

;*************************************************************************
;* subroutine: convert 16bit-word -> decimal -> ASCII -> plot HPGL 
;*************************************************************************
DecOut:
	rcall	Div10		; get "ones" digit
	ori	Temp1,$30	; convert to ASCII
	push	Temp1		; and save it
	rcall	Div10		; get "tens" digit
	ori	Temp1,$30	; convert to ASCII
	push	Temp1		; and save it
	rcall	Div10		; get "hundreds" digit
	ori	Temp1,$30	; convert to ASCII
	push	Temp1		; and save it
	rcall	Div10		; get "thousands" digit
	tst	Temp1		; omit leading zero
	breq	DecOu1		;
	ori	Temp1,$30	; convert to ASCII
	rcall	Serout		; print thousands	
DecOu1:
	pop	Temp1		; fetch hundreds
	rcall	Serout		; and print it 	
	pop	Temp1		; fetch tens
	rcall	Serout		; and print it 
	pop	Temp1		; fetch ones
	rcall	Serout		; and print it 	
	ret			; 

;*************************************************************************
;* subroutine: divide 16bit word in TempH/TempL by 10, 
;* results: quotient in TempH/TempL, remainder in Temp1 = decimal digit
;* taken from "Trampert, AVR-RISC-Mikrocontroller" 
;* Franzis-Verlag www.franzis.de ISBN 3772354742 pg. 327
;*************************************************************************
Div10:
	push	Cnt		;
	clr	Temp1		; 
	lsl	TempL		; input word is shifted out to the left
	rol	TempH		; filled up with zeroes from the right
	rol	Temp1		; 
	lsl	TempL		;
	rol	TempH		; most significant 3 bits are always < 10
	rol	Temp1		; so just shift out and in via Cy 3 times
	lsl	TempL		;
	rol	TempH		; 
	rol	Temp1		;
	ldi	Cnt,13		; start loop for next 13 bits
Div10a:
	lsl	TempL		;
	rol	TempH		; 
	rol	Temp1		;
	subi	Temp1,10	; remainder >10 ? -> subtract 10
	brlo	Div10b		; was <10, LSB of output word = 0
	inc	TempL		; LSB of output word = 1
	rjmp	Div10c
Div10b:
	subi	Temp1,-10	; back to old value <10
Div10c:
	dec	Cnt		; next bit
	brne	Div10a		;
	pop	Cnt		; 
	ret			;

;*************************************************************************
;* subroutine: output one character via asynchronous serial port
;* character in Temp1, Baudrate = 19200Bd 8N1 (Xon/Xoff ?)
;*************************************************************************
SerOut:
	out	UDR,Temp1	; start output
SerOu1:
	sbis	USR,6		; TxComplete-Flag set?
	rjmp	SerOu1		; not yet
	sbi	USR,6		; reset TXC-flag
	ret			;

;--------------------------- Interrupt Rotator switch --------------------
;
;*************************************************************************
;* Interrupt program for rotator switch
;*************************************************************************
ExtInt0:
	push	Temp1		; save Temp1
	in	Temp1,SREG	; save flag register
	push	Temp1
	in	Temp1,MCUCR	; Int0  _/~ or ~\_ ?
	sbrc	Temp1,0		; skip next line if  ~\_
	rjmp	UpA		; was _/~ so jump to UpA
	ori	Temp1,$01	; MCUCR-bit0 = 1
	out	MCUCR,Temp1	; Int0 next change will be _/~
	sbic	PinD,3		; skip next line if channel B=0
	rjmp	DecRot		; if B=1 and A=~\_ -> decrement
	rjmp	IncRot		; if B=0 and A=~\_ -> increment	
UpA:
	andi	Temp1,$FE	; MCUCR-bit0 = 0
	out	MCUCR,Temp1	; Int0 next change will be ~\_
	sbic	PinD,3		; skip next line if channel B=0
	rjmp	IncRot		; if B=1 and A=_/~ -> increment
	rjmp	DecRot		; if B=0 and A=_/~ -> decrement	

ExtInt1:
	push	Temp1		; save Temp1
	in	Temp1,SREG	; save flag register
	push	Temp1
	in	Temp1,MCUCR	; Int1  _/~ or ~\_ ?
	sbrc	Temp1,2		; skip next line when ~\_
	rjmp	UpB		; was _/~ so jump to UpB
	ori	Temp1,$04	; MCUCR-bit2 = 1
	out	MCUCR,Temp1	; Int1 next change will be _/~
	sbic	PinD,2		; skip next line if channel A=0
	rjmp	IncRot		; if A=1 and B=~\_ -> increment
	rjmp	DecRot		; if A=0 and B=~\_ -> decrement	
UpB:
	andi	Temp1,$FB	; MCUCR-bit2 = 0
	out	MCUCR,Temp1	; Int1 next change will be ~\_
	sbic	PinD,2		; skip next line if channel A=0
	rjmp	DecRot		; if A=1 and B=_/~ -> decrement
	rjmp	IncRot		; if A=0 and B=_/~ -> increment	

IncRot:
	mov	Temp1,RotVal	; max limit = +127 reached ?
	cpi	Temp1,$7F	; 
	breq	Exit		; yes, so don't increment
	inc	RotVal		; no, increment
	rjmp	Exit		;
DecRot:
	mov	Temp1,RotVal	; min limit = -128 reached ?
	cpi	Temp1,$80	; 
	breq	Exit		; yes, so don't decrement
	dec	RotVal		; no, decrement
Exit:
	pop	Temp1		; restore Flag register
	out	SREG,Temp1	;
	pop	Temp1		; restore Temp1
	reti

;--------------------------- Tables --------------------------------------
;
;*************************************************************************
;* Table:  LCD-controller initialization
;*************************************************************************
RegTbl:
	.db $04,$80,$83,$27,$EF,$00,$00,$00	; reg[01]-[08]
	.db $00,$00,$00,$00,$00,$00,$00,$00	; reg[09]-[10]
	.db $00,$00,$FF,$03,$00,$00,$00,$00	; reg[11]-[18]
	.db $00,$00,$00,$00			; reg[19]-[1C]
;*************************************************************************
;* Table: one line of text above scope-screen
;*************************************************************************
TxtTbl:
	.db "CENTER:1,50GHz  SPAN:1,4GHz  MARKER:0x00,00MHz  PRINT",$00

MnuTbl:
	.db 42,$A0,33,$70,51,$49,15,$10		; lenght+pos. of underline
;*************************************************************************
;* Table for HP-GL output to printer
;*************************************************************************
HpgTbl:
	.db $1B,"E",$1B,"%0B","IN"		; init Laserjet for HP-GL
	.db $3B,"SP1",$3B,"PW.25",$3B,"TR1"	; 
	.db $3B,"PU700,320PD700,4012,7100,4012"	; print frame
	.db ",7100,320,700,320",$0D,"PU700,4040"	;
	.db "LB Center: 1,5 GHz"		; print text
	.db "   Span: 1,4 GHz"			;
	.db "    Marker: 1707,35 MHz",$03	;
	.db "PU0,5400PD7800,5400",$00		; print border line
	.db $1B,"%0A",$1B,"E"			; switch back to PCLx

;*************************************************************************
;* Table for character display
;*************************************************************************
Tab5x8:
	.db $00,$00,$00,$00,$00,$00,$00,$00	; Space
	.db $20,$20,$20,$20,$00,$00,$20,$00	; !
	.db $50,$50,$50,$00,$00,$00,$00,$00	; "
	.db $50,$50,$F8,$50,$F8,$50,$50,$00	; #
	.db $20,$78,$A0,$70,$28,$F0,$20,$00	; $
	.db $C0,$C8,$10,$20,$40,$98,$18,$00	; %
	.db $60,$90,$A0,$40,$A8,$90,$68,$00	; &
	.db $60,$20,$40,$00,$00,$00,$00,$00	; 
	.db $10,$20,$40,$40,$40,$20,$10,$00	; (
	.db $40,$20,$10,$10,$10,$20,$40,$00	; )
	.db $00,$20,$A8,$70,$A8,$20,$00,$00	; *
	.db $00,$20,$20,$F8,$20,$20,$00,$00	; +
	.db $00,$00,$00,$00,$60,$20,$40,$00	; ,
	.db $00,$00,$00,$F8,$00,$00,$00,$00	; -
	.db $00,$00,$00,$00,$00,$60,$60,$00	; .
	.db $00,$08,$10,$20,$40,$80,$00,$00	; /
	.db $70,$88,$98,$A8,$C8,$88,$70,$00	; 0
	.db $20,$60,$20,$20,$20,$20,$70,$00	; 1
	.db $70,$88,$08,$10,$20,$40,$F8,$00	; 2
	.db $F8,$10,$20,$10,$08,$88,$70,$00	; 3
	.db $10,$30,$50,$90,$F8,$10,$10,$00	; 4
	.db $F8,$80,$F0,$08,$08,$88,$70,$00	; 5
	.db $30,$40,$80,$F0,$88,$88,$70,$00	; 6
	.db $F8,$08,$10,$20,$20,$20,$20,$00	; 7
	.db $70,$88,$88,$70,$88,$88,$70,$00	; 8
	.db $70,$88,$88,$78,$08,$10,$60,$00	; 9
	.db $00,$60,$60,$00,$60,$60,$00,$00	; :
	.db $00,$60,$60,$00,$60,$20,$40,$00	; ;
	.db $10,$20,$40,$80,$40,$20,$10,$00	; <
	.db $00,$00,$F8,$00,$F8,$00,$00,$00	; =
	.db $40,$20,$10,$08,$10,$20,$40,$00	; >
	.db $70,$88,$08,$10,$20,$00,$20,$00	; ?

	.db $70,$88,$08,$68,$A8,$A8,$70,$00	; @
	.db $70,$88,$88,$88,$F8,$88,$88,$00	; A
	.db $F0,$48,$48,$70,$48,$48,$F0,$00	; B
	.db $70,$88,$80,$80,$80,$88,$70,$00	; C
	.db $F0,$48,$48,$48,$48,$48,$F0,$00	; D
	.db $F8,$80,$80,$F8,$80,$80,$F8,$00	; E
	.db $F8,$80,$80,$F0,$80,$80,$80,$00	; F
	.db $70,$88,$80,$98,$88,$88,$78,$00	; G
	.db $88,$88,$88,$F8,$88,$88,$88,$00	; H
	.db $70,$20,$20,$20,$20,$20,$70,$00	; I
	.db $38,$10,$10,$10,$10,$90,$60,$00	; J
	.db $88,$90,$A0,$C0,$A0,$90,$88,$00	; K
	.db $80,$80,$80,$80,$80,$80,$F8,$00	; L
	.db $88,$D8,$A8,$A8,$88,$88,$88,$00	; M
	.db $88,$C8,$C8,$A8,$98,$98,$88,$00	; N
	.db $70,$88,$88,$88,$88,$88,$70,$00	; O
	.db $F0,$88,$88,$F0,$80,$80,$80,$00	; P
	.db $70,$88,$88,$88,$A8,$98,$68,$00	; Q
	.db $F0,$88,$88,$F0,$A0,$90,$88,$00	; R
	.db $70,$88,$80,$70,$08,$88,$70,$00	; S
	.db $F8,$20,$20,$20,$20,$20,$20,$00	; T
	.db $88,$88,$88,$88,$88,$88,$70,$00	; U
	.db $88,$88,$88,$88,$88,$50,$20,$00	; V
	.db $88,$88,$88,$A8,$A8,$D8,$88,$00	; W
	.db $88,$88,$50,$20,$50,$88,$88,$00	; X
	.db $88,$88,$88,$50,$20,$20,$20,$00	; Y
	.db $F8,$08,$10,$20,$40,$80,$F8,$00	; Z
	.db $70,$40,$40,$40,$40,$40,$70,$00	; [
	.db $00,$80,$40,$20,$10,$08,$00,$00	; \
	.db $70,$10,$10,$10,$10,$10,$70,$00	; ]
	.db $20,$50,$88,$00,$00,$00,$00,$00	; ^
	.db $00,$00,$00,$00,$00,$00,$F8,$00	; _

	.db $80,$40,$20,$00,$00,$00,$00,$00	; `
	.db $00,$00,$70,$08,$78,$88,$78,$00	; a
	.db $80,$80,$B0,$C8,$88,$88,$F0,$00	; b
	.db $00,$00,$70,$88,$80,$88,$70,$00	; c
	.db $08,$08,$68,$98,$88,$88,$78,$00	; d
	.db $00,$00,$70,$88,$F8,$80,$70,$00	; e
	.db $10,$28,$20,$70,$20,$20,$20,$00	; f
	.db $00,$68,$98,$98,$68,$08,$70,$00	; g
	.db $80,$80,$B0,$C8,$88,$88,$88,$00	; h
	.db $20,$00,$60,$20,$20,$20,$70,$00	; i
	.db $10,$00,$30,$10,$10,$90,$60,$00	; j
	.db $40,$40,$48,$50,$60,$50,$48,$00	; k
	.db $60,$20,$20,$20,$20,$20,$70,$00	; l
	.db $00,$00,$D0,$A8,$A8,$A8,$A8,$00	; m
	.db $00,$00,$B0,$C8,$88,$88,$88,$00	; n
	.db $00,$00,$70,$88,$88,$88,$70,$00	; o
	.db $00,$00,$B0,$C8,$C8,$B0,$80,$00	; p
	.db $00,$00,$68,$98,$98,$68,$08,$00	; q
	.db $00,$00,$B0,$C8,$80,$80,$80,$00	; r
	.db $00,$00,$78,$80,$70,$08,$F0,$00	; s
	.db $40,$40,$E0,$40,$40,$48,$30,$00	; t
	.db $00,$00,$90,$90,$90,$90,$68,$00	; u
	.db $00,$00,$90,$90,$90,$50,$20,$00	; v
	.db $00,$00,$88,$88,$A8,$A8,$50,$00	; w
	.db $00,$00,$88,$50,$20,$50,$88,$00	; x
	.db $00,$00,$88,$98,$68,$08,$70,$00	; y
	.db $00,$00,$F8,$10,$20,$40,$F8,$00	; z
	.db $10,$20,$20,$40,$20,$20,$10,$00	; {
	.db $20,$20,$20,$00,$20,$20,$20,$00	; |
	.db $40,$20,$20,$10,$20,$20,$40,$00	; }
	.db $40,$A8,$10,$00,$00,$00,$00,$00	; ~
	.db $F8,$F8,$F8,$F8,$F8,$F8,$F8,$00	; block


;*************************************************************************
;* Subroutine: display one byte as two ASCII-characters
;*************************************************************************
;HexOut:
;	push	TempL		; save high nibble
;	andi	TempL,$0F	; convert to ASCII
;	ori	TempL,$30	; x0..xF -> 30..3F
;	cpi	TempL,$3A	; is letter A..F ?
;	brlo	HexOu1		; no was number 0..9
;	subi	TempL,-7	; =addi TempL,+7 : $3A..$3F -> $41..$46
;HexOu1:
;	mov	LpmReg,TempL	; output to text line
;	rcall	CharOut
;	pop	TempL		; now low nibble
;	swap	TempL		; now high nibble
;	andi	TempL,$0F	; convert to ASCII
;	ori	TempL,$30	; x0..xF -> 30..3F
;	cpi	TempL,$3A	; is letter A..F ?
;	brlo	HexOu2		; no was number 0..9
;	subi	TempL,-7	; =addi TempL,+7 : $3A..$3F -> $41..$46
;HexOu2:
;	mov	LpmReg,TempL	; output to text line
;	rcall	CharOut
;	ret
;*************************************************************************
;* fill ram with gray scale
;*************************************************************************
;	ldi	XH,$40		; pixel data ram.$4000-$DFFF, visible -D5FF
;	ldi	XL,$00		;
;Fillmem:
;	ldi	Temp1,$FF	; pixel data $00...$FF
;Fillmm1
;	ldi	TempL,10	; 2pixel/byte*10rows*16graylevels = 320 pixel per line
;Fillmm2:
;	st	X+,Temp1	;
;	dec	TempL
;	brne	Fillmm2
;	subi	Temp1,$11
;	brcc	Fillmm1
;	cpi	XH,$D6		;
;	brne	FillMem

