;***************************************************************************
;* Weather satellite decoder and display using an                          *
;* Atmel AVR-microcontroller and Epson LC-Display-controller SED1374       *
;* for use with "1/4VGA"-Display (320*240 pixel) Samsung UG32-F05          *
;* Data input via Linear Technology LTC1298 12Bit ADC                      *
;* Data output via Linear Technology LTC1257 12Bit DAC                     *
;* Voltage regulators for +5V (LM2575T5.0), +33V (LT1172) -24V (79L24)     *
;* and +3,3V (LM1086-3.3) from Linear Technology and National Semiconductor*
;* PCB Layout : Eagle light 4.0                                            *
;* Christoph Kessler 2005                   (db1uq@t-online.de)            *
;***************************************************************************
.nolist
.include "8515def.inc"
.list
;***************************************************************************
;* register 0 ( special purpose: 16Bit-LPM-operation ):
;***************************************************************************
.def	LpmReg	= r00	; for table-operations
;***************************************************************************
;* register 1-15 ( no "immediate"-operations possible ):
;***************************************************************************
.def	Null	= r01	; always zero for easy 16 bit addition
.def	SavSrg	= r02	; save status during timer interrupt
.def	reg03	= r03	; 
.def	reg04	= r04	; 
.def	reg05	= r05	;
.def	reg06	= r06	; 
.def	reg05	= r07	;
.def	DblPix	= r08	; two 4bit-pixel in one byte
.def	VectL	= r09	; Vector lenght low
.def	VectM	= r10	; Vector lenght mid 
.def	VectH	= r11	; Vector lenght high 
.def	ADC3L	= r12	; ADC low  Byte oldest value
.def	ADC3H	= r13	; ADC high Byte oldest value
.def	ADC2L	= r14	; ADC low  Byte last value
.def	ADC2H	= r15	; ADC high Byte last value
;***************************************************************************
;* register 16-25 ( "immediate"-operations possible ):
;***************************************************************************
.def	ADC1L	= r16	; ADC low  Byte most recent value
.def	ADC1H	= r17	; ADC high Byte most recent value
.def	PixL	= r18	; pixel brightness 12bit low byte
.def	PixH	= r19	; pixel brightness 12bit high byte
.def	Cnt	= r20	; general temporary counter
.def	Temp1	= r21	; general temporary register
.def	Temp2	= r22	; general temporary register
.def	TempL	= r23	; general temporary register (low)
;***************************************************************************
;* register 24    ( also usable as 16Bit-register ) 
;***************************************************************************
.def	TempM	= r24	; general temporary register (mid)
.def	TempH	= r25	; general temporary register (high)
;***************************************************************************
;* register 26-31 ( 16Bit-register ) already defined in "inc"-file:
;***************************************************************************
;	XL	= r26	; pointer for character output low
;	XH	= r27	; pointer for character output high
;	YL	= r28	; pointer for display fading address low
;	YH	= r29	; pointer for display fading address high
;	ZL	= r30	; pointer for lpm operations low
;	ZH	= r31	; pointer for lpm operations high

;*************************************************************************
;* constants :
;*************************************************************************
; Port B connections:
.equ	SCK	= 7	; PortB Bit7 = input  serial programming clock
.equ	MISO	= 6	; PortB Bit6 = output serial programming data out
.equ	MOSI	= 5	; PortB Bit5 = input  serial programming data in
.equ	CsAdc	= 4	; PortB Bit4 = output chip select ADC
.equ	SDat	= 3	; PortB Bit3 = bidir  serial data ADC and DAC
.equ	Click	= 2	; PortB Bit2 = input  click switch input
.equ	T1	= 1	; PortB Bit1 = input  Timer 1 from divider
.equ	T0	= 0	; PortB Bit0 = output Flash-RAM /CS
; Port D connections:
.equ	RD	= 7	; PortD Bit7 = output read LCD controller
.equ	WR	= 6	; PortD Bit6 = output write LCD controller
.equ	SClk	= 5	; PortD Bit5 = output serial clock ADC and DAC
.equ	LdDac	= 4	; PortD Bit4 = output load DAC
.equ	IN1	= 3	; PortD Bit3 = input  rotator switch channel B
.equ	IN0	= 2	; PortD Bit2 = input  rotator switch channel A
.equ	TXD	= 1	; PortD Bit1 = output UART out
.equ	RXD	= 0	; PortD Bit0 = input  UART in

;--------------------------- Initialization ------------------------------

;*************************************************************************
;* reset/interrupt-vectors:
;*************************************************************************
.cseg
	rjmp	Initial		; after RESET to main program
	nop			; rjmp ExtInt0
	nop			; rjmp ExtInt1
	nop			; Timer1_capture
	nop			; Timer1_compareA
	nop			; Timer1_compareB
	nop			; Timer1_overflow
	rjmp	Int9k6		; Timer0_overflow
;	nop			; SPI transfer
;	nop			; UART Rx
;	nop			; UART Empty
;	nop			; UART Tx
;	nop			; Analog comparator
;*************************************************************************
;* after reset: initialise stack,ports,ext.SRAM,interrupts 
;*************************************************************************
Initial: 
	ldi	Temp1,Low(RamEnd)
	out	spl,Temp1	;
	ldi	Temp1,High(RamEnd)
	out	sph,Temp1	; stack initialised
	clr	Null		; always zero (useful for 16+8 bit add.)
	out	PortA,Null	;
	out	PortB,Null	;
	out	PortC,Null	;
	out	PortD,Null	;
	out	DDRA,Null	; (dont care,display-RAM)
	out	DDRC,Null	; (dont care,display-RAM)
	ldi	Temp1,$11	; PortB direction = % 0001 0001
	out	DDRB,Temp1	; 
	ldi	Temp1,$F2	; PortD direction = % 1111 0010
	out	DDRD,Temp1	;
	ldi	Temp1,$CA	; ext.SRAM enable, add.waitstate,Int0/1 _/~
	out	MCUCR,Temp1	; MicroControllerUnit Control Reg.
	sbi	PortB,CsAdc	; ADC-/Enable _/~
	sbi	DDRB,SDat	; SDat = output
;*************************************************************************
;* initialize LCD-controller registers
;*************************************************************************
	ldi	XH,$3F		; LCD-controller register 01 = Adr.$3FE1
	ldi	XL,$E1		;
	ldi	Cnt,28		; initialize 28 registers
	ldi	ZH,high(RegTbl)	; 
	ldi	ZL,low(RegTbl)	;
	lsl	ZL		; shift 1 left (16Bit memory addressing)
	rol	ZH		;
RgLoop:
	lpm
	st	X+,LpmReg	;
	adiw	ZL,1		; increment pointer
	dec	Cnt
	brne	RgLoop
;*************************************************************************
;* fill ram with gray scale
;*************************************************************************
	ldi	XH,$40		; pixel data ram.$4000-$DFFF, visible -D5FF
	clr	XL		;
Fillmem:
	ldi	Temp1,$FF	; pixel data $00...$FF
Fillmm1:
	ldi	TempL,10	; 2pixel/byte*10rows*16graylevels = 320 pixel per line
Fillmm2:
	st	X+,Temp1	;
	dec	TempL
	brne	Fillmm2
	subi	Temp1,$11
	brcc	Fillmm1
	cpi	XH,$D6		;
	brne	FillMem
	ldi	XH,$40		; pixel data ram.$4000-$DFFF, visible -D5FF
	clr	XL		;
	ldi	Cnt,30
;*************************************************************************
;* start timer interrupt every 1/9600 seconds
;*************************************************************************
	ldi	Temp1,(255-104)	; up-counter 104sec=1/9600Hz
	out	TCNT0,Temp1	; into Timer/Counter0-Control-Register
	ldi	Temp1,$02	; prescaler/8 ->  1sec/clock
	out	TCCR0,Temp1	; into Timer/Counter0-Register
	out	TIMSK,Temp1	; into TimerInterrupt-Mask-Register
	sei			; enable interrupts

;--------------------------- Main Program loop ---------------------------
Endless:
	rjmp	Endless		; 


;--------------------------- Timer interrupt -----------------------------

;*************************************************************************
;* Timer interrupt routine every 1/9600 seconds = 104.17 usec
;* simultaneously reads analog-digital converter LTC1298
;* and computes vector lenght from last two measurements. 
;* Squareroot of (X^2 + Y2) is approximated by :
;* (larger one of X,Y) + 106/256 * (smaller one of X,Y) 
;* 106/256 is an approximation of Sqrt(2)-1 = 0.4142
;* so the circle is approximated by its inscribed eight corner area
;* this interrupt consumes about 75 percent of total processor time
;* AD-conversion parts are marked with "****"
;*************************************************************************
Int9k6:
	in	SavSrg,SREG	; (1) save status register
	sbi	PortB,T0	; (2) Test:Ausgang whrend Interrupt high
	nop
	nop
	nop
	ldi	Temp1,255-102	; (1) up-counter 104sec=1/9600Hz
	out	TCNT0,Temp1	; (1) into Timer/Counter0-Control-Register
;* Startbit *****************************
	cbi	PortD,SClk	; (2)	*
	sbi	DDRB,SDat	; (2)	*
	sbi	PortB,SDat	; (2)	*
	cbi	PortB,CsAdc	; (2)	*
;****************************************
	cp	ADC3H,ADC2H	; (1) find larger value
	brlo	Lo3		; (1+1)
	brne	Hi3		; (1+1) if equal
	cp	ADC3L,ADC2L	; (1) compare low bytes
	brlo	Lo3		; (1+1)
Hi3:
	set			; (1) T-flag=H -> ADC3 >= ADC2
	mov	TempM,ADC2H	; (1)
	mov	TempL,ADC2L	; (1)
	rjmp	By106		; (2)
Lo3:
	clt			; (1) T-flag=L -> ADC3 < ADC2
	mov	TempM,ADC3H	; (1)
	mov	TempL,ADC3L	; (1)
By106:
;****************************************
	sbi	PortD,SClk	; (2)	*
;****************************************
	clr	TempH		; (1) multiply smaller value by 106
	lsl	TempL		; (1) 106 = 2+8+32+64
	rol	TempM		; (1) = "*2"
	mov	VectL,TempL	; (1)
	mov	VectM,TempM	; (1) add to vector
	clr	VectH		; (1)
	lsl	TempL		; (1)
	rol	TempM		; (1)
	lsl	TempL		; (1)
	rol	TempM		; (1) = "*8"
	add	VectL,TempL	; (1)
	adc	VectM,TempM	; (1) add to vector
	lsl	TempL		; (1)
	rol	TempM		; (1)
;* SGL/Diff *****************************
	cbi	PortD,SClk	; (2)	*	
;****************************************
	lsl	TempL		; (1)
	rol	TempM		; (1) = "*32", high byte = 0
	add	VectL,TempL	; (1)
	adc	VectM,TempM	; (1) add to vector
	lsl	TempL		; (1)
	rol	TempM		; (1) = "*64"
	rol	TempH		; (1) now we need a 3rd byte
;****************************************
	sbi	PortD,SClk	; (2)	*
;****************************************
	add	VectL,TempL	; (1)
	adc	VectM,TempM	; (1) add to vector
	adc	VectH,TempH	; (1) 
	brts	x		; (1+1)
	add	VectM,ADC2L	; (1)
	adc	VectH,ADC2H	; (1)
	rjmp	y		; (2)
x:
	add	VectM,ADC3L	; (1)
	adc	VectH,ADC3H	; (1)
	nop			; (1)
y:
	mov	ADC3L,ADC2L	; (1)
	mov	ADC3H,ADC2H	; (1)
	nop			; (1)
	add	PixL,VectM	; (1) add 15 12bit words for 1 pixel
	adc	PixH,VectH	; (1)
	dec	Cnt		; (1) 2*15 pixel over ?
;* Eingang2 *****************************
	cbi	PortD,SClk	; (2)	*
;****************************************
	brne	Pixel2		; (1+1) not yet
	clr	PixL		; (1) discard lowest 8 bit
	andi	PixH,$F0	; (1) mask upper 4 bit
	swap	PixH		; (1)
	or	DblPix,PixH	; (1) put 2 pixel into 1 Byte
	com	DblPix		; (1) negative color
	st	X+,DblPix	; (2) store into pixel ram
	cpi	XH,$D6		; (1) end of pixel ram reached ?
	brne	Pixel1		; (1+1)
	ldi	XH,$40		; (1) goto start of pixel ram =$4000
Pixel1:
	clr	PixH		; (1)
	ldi	Cnt,30		; (1)
	rjmp	Pixel5		; (2)
Pixel2:
	cpi	Cnt,15		; (1) 15 pixel over?
	brne	Pixel3		; (1+1)
	clr	PixL		; (1) discard lowest 8 bit
	andi	PixH,$F0	; (1) mask upper 4 bit
	mov	DblPix,PixH	; (1)
	clr	PixH		; (1)
	rjmp	Pixel4		; (2)
Pixel3:
	nop			; (1)
	nop			; (1)
	nop			; (1)
	nop			; (1)
	nop			; (1)
Pixel4:
	nop			; (1)
	nop			; (1)
	nop			; (1)
Pixel5:
	nop			; (1)
;****************************************
	sbi	PortD,SClk	; (2)	*
;****************************************

	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;* MSB first ****************************
	cbi	PortD,SClk	; (2)	*
;****************************************
	nop			; (14)
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;****************************************
	sbi	PortD,SClk	; (2)	*
;****************************************
	nop			; (12)
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;* Null-Bit *****************************
	cbi	DDRB,SDat	; (2)	*
	cbi	PortD,SClk	; (2)	*
;****************************************
	nop			; (14) 
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;****************************************
	sbi	PortD,SClk	; (2)	*
;****************************************
	nop			; (13)
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;* Bit 11 *******************************
	cbi	PortD,SClk	; (2)	*
;****************************************
	nop			; (14) 
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;****************************************
	sbi	PortD,SClk	; (2)  	*
	clr	ADC1H		; (1)	*
	clc			; (1)  	*
	sbic	PinB,SDat	; (1+1)	* 
	sec			; (1)	*
	rol	ADC1H		; (1)	*
;****************************************
	nop			; (9)
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;* Bit 10 *******************************
	cbi	PortD,SClk	; (2)	*
;****************************************
	nop			; (14) 
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;****************************************
	sbi	PortD,SClk	; (2)  	*
	clc			; (1)  	*
	sbic	PinB,SDat	; (1+1)	* 
	sec			; (1)	*
	rol	ADC1H		; (1)	*
;****************************************
	nop			; (10)
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;* Bit 9 ********************************
	cbi	PortD,SClk	; (2)	*
;****************************************
	nop			; (14) 
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;****************************************
	sbi	PortD,SClk	; (2)  	*
	clc			; (1)  	*
	sbic	PinB,SDat	; (1+1)	* 
	sec			; (1)	*
	rol	ADC1H		; (1)	*
;****************************************
	nop			; (10)
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;* Bit 8 ********************************
	cbi	PortD,SClk	; (2)	*
;****************************************
	nop			; (14) 
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;****************************************
	sbi	PortD,SClk	; (2)  	*
	clc			; (1)  	*
	sbic	PinB,SDat	; (1+1)	* 
	sec			; (1)	*
	rol	ADC1H		; (1)	*
;****************************************
	nop			; (10)
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;* Bit 7 ********************************
	cbi	PortD,SClk	; (2)	*
;****************************************
	nop			; (14) 
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;****************************************
	sbi	PortD,SClk	; (2)  	*
	clc			; (1)  	*
	sbic	PinB,SDat	; (1+1)	* 
	sec			; (1)	*
	rol	ADC1L		; (1)	*
;****************************************
	nop			; (10)
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;* Bit 6 ********************************
	cbi	PortD,SClk	; (2)	*
;****************************************
	nop			; (14) 
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;****************************************
	sbi	PortD,SClk	; (2)  	*
	clc			; (1)  	*
	sbic	PinB,SDat	; (1+1)	* 
	sec			; (1)	*
	rol	ADC1L		; (1)	*
;****************************************
	nop			; (10)
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;* Bit 5 ********************************
	cbi	PortD,SClk	; (2)	*
;****************************************
	nop			; (14) 
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;****************************************
	sbi	PortD,SClk	; (2)  	*
	clc			; (1)  	*
	sbic	PinB,SDat	; (1+1)	* 
	sec			; (1)	*
	rol	ADC1L		; (1)	*
;****************************************
	nop			; (10)
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;* Bit 4 ********************************
	cbi	PortD,SClk	; (2)	*
;****************************************
	nop			; (14) 
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;****************************************
	sbi	PortD,SClk	; (2)  	*
	clc			; (1)  	*
	sbic	PinB,SDat	; (1+1)	* 
	sec			; (1)	*
	rol	ADC1L		; (1)	*
;****************************************
	nop			; (10)
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;* Bit 3 ********************************
	cbi	PortD,SClk	; (2)	*
;****************************************
	nop			; (14) 
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;****************************************
	sbi	PortD,SClk	; (2)  	*
	clc			; (1)  	*
	sbic	PinB,SDat	; (1+1)	* 
	sec			; (1)	*
	rol	ADC1L		; (1)	*
;****************************************
	nop			; (10)
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;* Bit 2 ********************************
	cbi	PortD,SClk	; (2)	*
;****************************************
	nop			; (14) 
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;****************************************
	sbi	PortD,SClk	; (2)  	*
	clc			; (1)  	*
	sbic	PinB,SDat	; (1+1)	* 
	sec			; (1)	*
	rol	ADC1L		; (1)	*
;****************************************
	nop			; (10)
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;* Bit 1 ********************************
	cbi	PortD,SClk	; (2)	*
;****************************************
	nop			; (14) 
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;****************************************
	sbi	PortD,SClk	; (2)  	*
	clc			; (1)  	*
	sbic	PinB,SDat	; (1+1)	* 
	sec			; (1)	*
	rol	ADC1L		; (1)	*
;****************************************
	nop			; (10)
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;* Bit 0 ********************************
	cbi	PortD,SClk	; (2)	*
;****************************************
	nop			; (14) 
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;****************************************
	sbi	PortD,SClk	; (2)  	*
	clc			; (1)  	*
	sbic	PinB,SDat	; (1+1)	* 
	sec			; (1)	*
	rol	ADC1L		; (1)	*
;****************************************
	nop			; (10)
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
;****************************************
	sbi	PortB,CsAdc	; (2)	*
	sbi	DDRB,SDat	; (2)	*
;****************************************
	cpi	ADC1H,$08	; (1) get absolute value of 12 bit word
	brsh	Absol		; (1+1) dont change if = $0800...0FFF
	com	ADC1H		; (1) complement if = $0000...07FF
	com	ADC1L		; (1) 
Absol:
	andi	ADC1H,$07	; (1) clear 5 MSBs including sign bit


	lsl	VectM		; (1)
	rol	VectH		; (1)
	lsl	VectM		; (1)
	rol	VectH		; (1)
	lsl	VectM		; (1)
	rol	VectH		; (1)
	lsl	VectM		; (1)
	rol	VectH		; (1)


;* DA-Conversion *********************************************************
	cbi	PortD,SClk	; (2) DAC-clock ~\_
	lsl	VectH		; (1)
	brcc	DAC11L		; (1+1) one bit via Cy into DAC
	sbi	PortB,SDat	; (2) SDat=H
	brcs	DAC11H		; (1+1)
DAC11L:
	cbi	PortB,SDat	; (2) SDat=L
DAC11H:
	sbi	PortD,SClk	; (2) DAC-clock _/~
	lsl	VectH		; (1)
	cbi	PortD,SClk	; (2) DAC-clock ~\_
	brcc	DAC10L		; (1+1) one bit via Cy into DAC
	sbi	PortB,SDat	; (2) SDat=H
	brcs	DAC10H		; (1+1)
DAC10L:
	cbi	PortB,SDat	; (2) SDat=L
DAC10H:
	sbi	PortD,SClk	; (2) DAC-clock _/~
	lsl	VectH		; (1)
	cbi	PortD,SClk	; (2) DAC-clock ~\_
	brcc	DAC9L		; (1+1) one bit via Cy into DAC
	sbi	PortB,SDat	; (2) SDat=H
	brcs	DAC9H		; (1+1)
DAC9L:
	cbi	PortB,SDat	; (2) SDat=L
DAC9H:
	sbi	PortD,SClk	; (2) DAC-clock _/~
	lsl	VectH		; (1)
	cbi	PortD,SClk	; (2) DAC-clock ~\_
	brcc	DAC8L		; (1+1) one bit via Cy into DAC
	sbi	PortB,SDat	; (2) SDat=H
	brcs	DAC8H		; (1+1)
DAC8L:
	cbi	PortB,SDat	; (2) SDat=L
DAC8H:
	sbi	PortD,SClk	; (2) DAC-clock _/~
	lsl	VectH		; (1)
	cbi	PortD,SClk	; (2) DAC-clock ~\_
	brcc	DAC7L		; (1+1) one bit via Cy into DAC
	sbi	PortB,SDat	; (2) SDat=H
	brcs	DAC7H		; (1+1)
DAC7L:
	cbi	PortB,SDat	; (2) SDat=L
DAC7H:
	sbi	PortD,SClk	; (2) DAC-clock _/~
	lsl	VectH		; (1)
	cbi	PortD,SClk	; (2) DAC-clock ~\_
	brcc	DAC6L		; (1+1) one bit via Cy into DAC
	sbi	PortB,SDat	; (2) SDat=H
	brcs	DAC6H		; (1+1)
DAC6L:
	cbi	PortB,SDat	; (2) SDat=L
DAC6H:
	sbi	PortD,SClk	; (2) DAC-clock _/~
	lsl	VectH		; (1) max clock freq. 1,4MHz!
	cbi	PortD,SClk	; (2) DAC-clock ~\_
	brcc	DAC5L		; (1+1) one bit via Cy into DAC
	sbi	PortB,SDat	; (2) SDat=H
	brcs	DAC5H		; (1+1)
DAC5L:
	cbi	PortB,SDat	; (2) SDat=L
DAC5H:
	sbi	PortD,SClk	; (1) DAC-clock _/~
	lsl	VectH		; (1) max clock freq. 1,4MHz!
	cbi	PortD,SClk	; (2) DAC-clock ~\_
	brcc	DAC4L		; (1+1) one bit via Cy into DAC
	sbi	PortB,SDat	; (2) SDat=H
	brcs	DAC4H		; (1+1)
DAC4L:
	cbi	PortB,SDat	; (2) SDat=L
DAC4H:
	sbi	PortD,SClk	; (2) DAC-clock _/~
	lsl	VectM		; (1) max clock freq. 1,4MHz!
	cbi	PortD,SClk	; (2) DAC-clock ~\_
	brcc	DAC3L		; (1+1) one bit via Cy into DAC
	sbi	PortB,SDat	; (2) SDat=H
	brcs	DAC3H		; (1+1)
DAC3L:
	cbi	PortB,SDat	; (2) SDat=L
DAC3H:
	sbi	PortD,SClk	; (2) DAC-clock _/~
	lsl	VectM		; (1) 
	cbi	PortD,SClk	; (2) DAC-clock ~\_
	brcc	DAC2L		; (1+1) one bit via Cy into DAC
	sbi	PortB,SDat	; (2) SDat=H
	brcs	DAC2H		; (1+1)
DAC2L:
	cbi	PortB,SDat	; (2) SDat=L
DAC2H:
	sbi	PortD,SClk	; (2) DAC-clock _/~
	lsl	VectM		; (1) 
	cbi	PortD,SClk	; (2) DAC-clock ~\_
	brcc	DAC1L		; (1+1) one bit via Cy into DAC
	sbi	PortB,SDat	; (2) SDat=H
	brcs	DAC1H		; (1+1)
DAC1L:
	cbi	PortB,SDat	; (2) SDat=L
DAC1H:
	sbi	PortD,SClk	; (2) DAC-clock _/~
	lsl	VectM		; (1) 
	cbi	PortD,SClk	; (2) DAC-clock ~\_
	brcc	DAC0L		; (1+1) one bit via Cy into DAC
	sbi	PortB,SDat	; (2) SDat=H
	brcs	DAC0H		; (1+1)
DAC0L:
	cbi	PortB,SDat	; (2) SDat=L
DAC0H:
	sbi	PortD,SClk	; (2) DAC-clock _/~
	cbi	PortD,LdDac	; (2) DA-converter /load ~\_
	mov	ADC2L,ADC1L	; (1) 
	mov	ADC2H,ADC1H	; (1)
	sbi	PortD,LdDac	; (2) DA-converter /load _/~
	cbi	PortB,T0	; (2) Test:Ausgang whrend Interrupt high
	out	SREG,SavSrg	; (1) fetch status register
	reti			; (4)
;*************************************************************************

;--------------------------- Tables --------------------------------------
;
;*************************************************************************
;* Table:  LCD-controller initialization
;*************************************************************************
RegTbl:
	.db $04,$80,$83,$27,$EF,$00,$00,$00	; reg[01]-[08]
	.db $00,$00,$00,$00,$00,$00,$00,$00	; reg[09]-[10]
	.db $00,$00,$FF,$03,$00,$00,$00,$00	; reg[11]-[18]
	.db $00,$00,$00,$00			; reg[19]-[1C]

